@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>. 
  *
- * Copyright 2008-2020 The OpenLDAP Foundation.
+ * Copyright 2008-2022 The OpenLDAP Foundation.
  * Portions Copyright 2008 by Howard Chu, Symas Corp.
  * All rights reserved.
  *
@@ -225,7 +225,7 @@ static int write_passwd(nssov_passwd_cbp *cbp,Entry *entry)
 		if (!a)
 		{
 			Debug(LDAP_DEBUG_ANY,"passwd entry %s does not contain %s value\n",
-				entry->e_name.bv_val, cbp->mi->mi_attrs[UID_KEY].an_desc->ad_cname.bv_val,0);
+				entry->e_name.bv_val, cbp->mi->mi_attrs[UID_KEY].an_desc->ad_cname.bv_val );
 			return 0;
 		}
 		names = a->a_vals;
@@ -266,7 +266,7 @@ static int write_passwd(nssov_passwd_cbp *cbp,Entry *entry)
         if ( !a )
 		{
 			Debug(LDAP_DEBUG_ANY,"passwd entry %s does not contain %s value\n",
-				entry->e_name.bv_val, cbp->mi->mi_attrs[UIDN_KEY].an_desc->ad_cname.bv_val,0);
+				entry->e_name.bv_val, cbp->mi->mi_attrs[UIDN_KEY].an_desc->ad_cname.bv_val );
 			return 0;
 		}
 		uids = a->a_vals;
@@ -282,19 +282,19 @@ static int write_passwd(nssov_passwd_cbp *cbp,Entry *entry)
 	if (!a)
 	{
 		Debug(LDAP_DEBUG_ANY,"passwd entry %s does not contain %s value\n",
-			entry->e_name.bv_val, cbp->mi->mi_attrs[GIDN_KEY].an_desc->ad_cname.bv_val,0);
+			entry->e_name.bv_val, cbp->mi->mi_attrs[GIDN_KEY].an_desc->ad_cname.bv_val );
 		return 0;
 	}
 	else if (a->a_numvals != 1)
 	{
 		Debug(LDAP_DEBUG_ANY,"passwd entry %s contains multiple %s values\n",
-			entry->e_name.bv_val, cbp->mi->mi_attrs[GIDN_KEY].an_desc->ad_cname.bv_val,0);
+			entry->e_name.bv_val, cbp->mi->mi_attrs[GIDN_KEY].an_desc->ad_cname.bv_val );
 	}
 	gid=(gid_t)strtol(a->a_vals[0].bv_val,&tmp,0);
 	if ((a->a_vals[0].bv_val[0]=='\0')||(*tmp!='\0'))
 	{
 		Debug(LDAP_DEBUG_ANY,"passwd entry %s contains non-numeric %s value\n",
-			entry->e_name.bv_val, cbp->mi->mi_attrs[GIDN_KEY].an_desc->ad_cname.bv_val,0);
+			entry->e_name.bv_val, cbp->mi->mi_attrs[GIDN_KEY].an_desc->ad_cname.bv_val );
 		return 0;
 	}
 	/* get the gecos for this entry (fall back to cn) */
@@ -322,7 +322,7 @@ static int write_passwd(nssov_passwd_cbp *cbp,Entry *entry)
 	if (!a)
 	{
 		Debug(LDAP_DEBUG_ANY,"passwd entry %s does not contain %s value\n",
-			entry->e_name.bv_val, cbp->mi->mi_attrs[DIR_KEY].an_desc->ad_cname.bv_val,0);
+			entry->e_name.bv_val, cbp->mi->mi_attrs[DIR_KEY].an_desc->ad_cname.bv_val );
 		homedir=default_passwd_homeDirectory;
 	}
 	else
@@ -330,7 +330,7 @@ static int write_passwd(nssov_passwd_cbp *cbp,Entry *entry)
 		if (a->a_numvals > 1)
 		{
 			Debug(LDAP_DEBUG_ANY,"passwd entry %s contains multiple %s values\n",
-				entry->e_name.bv_val, cbp->mi->mi_attrs[DIR_KEY].an_desc->ad_cname.bv_val,0);
+				entry->e_name.bv_val, cbp->mi->mi_attrs[DIR_KEY].an_desc->ad_cname.bv_val );
 		}
 		homedir=a->a_vals[0];
 		if (homedir.bv_val[0]=='\0')
@@ -347,7 +347,7 @@ static int write_passwd(nssov_passwd_cbp *cbp,Entry *entry)
 		if (a->a_numvals > 1)
 		{
 			Debug(LDAP_DEBUG_ANY,"passwd entry %s contains multiple %s values\n",
-				entry->e_name.bv_val, cbp->mi->mi_attrs[SHL_KEY].an_desc->ad_cname.bv_val,0);
+				entry->e_name.bv_val, cbp->mi->mi_attrs[SHL_KEY].an_desc->ad_cname.bv_val );
 		}
 		shell=a->a_vals[0];
 		if (shell.bv_val[0]=='\0')
@@ -359,7 +359,7 @@ static int write_passwd(nssov_passwd_cbp *cbp,Entry *entry)
 		if (!isvalidusername(&names[i]))
 		{
 			Debug(LDAP_DEBUG_ANY,"nssov: passwd entry %s contains invalid user name: \"%s\"\n",
-				entry->e_name.bv_val,names[i].bv_val,0);
+				entry->e_name.bv_val,names[i].bv_val );
 		}
 		else
 		{
@@ -399,11 +399,11 @@ NSSOV_HANDLE(
 	cbp.name.bv_len = tmpint32;
 	cbp.name.bv_val = cbp.buf;
 	if (!isvalidusername(&cbp.name)) {
-		Debug(LDAP_DEBUG_ANY,"nssov_passwd_byname(%s): invalid user name\n",cbp.name.bv_val,0,0);
+		Debug(LDAP_DEBUG_ANY,"nssov_passwd_byname(%s): invalid user name\n",cbp.name.bv_val);
 		return -1;
 	}
 	BER_BVZERO(&cbp.id); ,
-	Debug(LDAP_DEBUG_TRACE,"nssov_passwd_byname(%s)\n",cbp.name.bv_val,0,0);,
+	Debug(LDAP_DEBUG_TRACE,"nssov_passwd_byname(%s)\n",cbp.name.bv_val);,
 	NSLCD_ACTION_PASSWD_BYNAME,
 	nssov_filter_byname(cbp.mi,UID_KEY,&cbp.name,&filter)
 )
@@ -418,7 +418,7 @@ NSSOV_HANDLE(
 	cbp.id.bv_val = cbp.buf;
 	cbp.id.bv_len = snprintf(cbp.buf,sizeof(cbp.buf),"%d",uid);
 	BER_BVZERO(&cbp.name);,
-	Debug(LDAP_DEBUG_TRACE,"nssov_passwd_byuid(%s)\n",cbp.id.bv_val,0,0);,
+	Debug(LDAP_DEBUG_TRACE,"nssov_passwd_byuid(%s)\n",cbp.id.bv_val);,
 	NSLCD_ACTION_PASSWD_BYUID,
 	nssov_filter_byid(cbp.mi,UIDN_KEY,&cbp.id,&filter)
 )
@@ -429,7 +429,7 @@ NSSOV_HANDLE(
 	/* no parameters to read */
 	BER_BVZERO(&cbp.name);
 	BER_BVZERO(&cbp.id);,
-	Debug(LDAP_DEBUG_TRACE,"nssov_passwd_all()\n",0,0,0);,
+	Debug(LDAP_DEBUG_TRACE,"nssov_passwd_all()\n");,
 	NSLCD_ACTION_PASSWD_ALL,
 	(filter=cbp.mi->mi_filter,0)
 )
