@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2020 The OpenLDAP Foundation.
+ * Copyright 1998-2022 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -362,7 +362,7 @@ ldap_pvt_put_filter( BerElement *ber, const char *str_in )
 	 * Note: tags in a CHOICE are always explicit
 	 */
 
-	Debug( LDAP_DEBUG_TRACE, "put_filter: \"%s\"\n", str_in, 0, 0 );
+	Debug1( LDAP_DEBUG_TRACE, "put_filter: \"%s\"\n", str_in );
 
 	freeme = LDAP_STRDUP( str_in );
 	if( freeme == NULL ) return LDAP_NO_MEMORY;
@@ -380,8 +380,7 @@ ldap_pvt_put_filter( BerElement *ber, const char *str_in )
 
 			switch ( *str ) {
 			case '&':
-				Debug( LDAP_DEBUG_TRACE, "put_filter: AND\n",
-				    0, 0, 0 );
+				Debug0( LDAP_DEBUG_TRACE, "put_filter: AND\n" );
 
 				str = put_complex_filter( ber, str,
 				    LDAP_FILTER_AND, 0 );
@@ -394,8 +393,7 @@ ldap_pvt_put_filter( BerElement *ber, const char *str_in )
 				break;
 
 			case '|':
-				Debug( LDAP_DEBUG_TRACE, "put_filter: OR\n",
-				    0, 0, 0 );
+				Debug0( LDAP_DEBUG_TRACE, "put_filter: OR\n" );
 
 				str = put_complex_filter( ber, str,
 				    LDAP_FILTER_OR, 0 );
@@ -408,8 +406,7 @@ ldap_pvt_put_filter( BerElement *ber, const char *str_in )
 				break;
 
 			case '!':
-				Debug( LDAP_DEBUG_TRACE, "put_filter: NOT\n",
-				    0, 0, 0 );
+				Debug0( LDAP_DEBUG_TRACE, "put_filter: NOT\n" );
 
 				str = put_complex_filter( ber, str,
 				    LDAP_FILTER_NOT, 0 );
@@ -426,8 +423,7 @@ ldap_pvt_put_filter( BerElement *ber, const char *str_in )
 				goto done;
 
 			default:
-				Debug( LDAP_DEBUG_TRACE, "put_filter: simple\n",
-				    0, 0, 0 );
+				Debug0( LDAP_DEBUG_TRACE, "put_filter: simple\n" );
 
 				balance = 1;
 				escape = 0;
@@ -472,8 +468,7 @@ ldap_pvt_put_filter( BerElement *ber, const char *str_in )
 			break;
 
 		case /*'('*/ ')':
-			Debug( LDAP_DEBUG_TRACE, "put_filter: end\n",
-				0, 0, 0 );
+			Debug0( LDAP_DEBUG_TRACE, "put_filter: end\n" );
 			if ( ber_printf( ber, /*"["*/ "]" ) == -1 ) {
 				rc = -1;
 				goto done;
@@ -487,8 +482,7 @@ ldap_pvt_put_filter( BerElement *ber, const char *str_in )
 			break;
 
 		default:	/* assume it's a simple type=value filter */
-			Debug( LDAP_DEBUG_TRACE, "put_filter: default\n",
-				0, 0, 0 );
+			Debug0( LDAP_DEBUG_TRACE, "put_filter: default\n" );
 			next = strchr( str, '\0' );
 			if ( put_simple_filter( ber, str ) == -1 ) {
 				rc = -1;
@@ -518,8 +512,8 @@ put_filter_list( BerElement *ber, char *str, ber_tag_t tag )
 	char	*next = NULL;
 	char	save;
 
-	Debug( LDAP_DEBUG_TRACE, "put_filter_list \"%s\"\n",
-		str, 0, 0 );
+	Debug1( LDAP_DEBUG_TRACE, "put_filter_list \"%s\"\n",
+		str );
 
 	while ( *str ) {
 		while ( *str && LDAP_SPACE( (unsigned char) *str ) ) {
@@ -558,8 +552,8 @@ put_simple_filter(
 	ber_tag_t	ftype;
 	int		rc = -1;
 
-	Debug( LDAP_DEBUG_TRACE, "put_simple_filter: \"%s\"\n",
-		str, 0, 0 );
+	Debug1( LDAP_DEBUG_TRACE, "put_simple_filter: \"%s\"\n",
+		str );
 
 	str = LDAP_STRDUP( str );
 	if( str == NULL ) return -1;
@@ -724,8 +718,8 @@ put_substring_filter( BerElement *ber, char *type, char *val, char *nextstar )
 	int gotstar = 0;
 	ber_tag_t	ftype = LDAP_FILTER_SUBSTRINGS;
 
-	Debug( LDAP_DEBUG_TRACE, "put_substring_filter \"%s=%s\"\n",
-		type, val, 0 );
+	Debug2( LDAP_DEBUG_TRACE, "put_substring_filter \"%s=%s\"\n",
+		type, val );
 
 	if ( ber_printf( ber, "t{s{" /*"}}"*/, ftype, type ) == -1 ) {
 		return -1;
@@ -810,7 +804,7 @@ put_vrFilter( BerElement *ber, const char *str_in )
 	 * (Source: RFC 3876)
 	 */
 
-	Debug( LDAP_DEBUG_TRACE, "put_vrFilter: \"%s\"\n", str_in, 0, 0 );
+	Debug1( LDAP_DEBUG_TRACE, "put_vrFilter: \"%s\"\n", str_in );
 
 	freeme = LDAP_STRDUP( str_in );
 	if( freeme == NULL ) return LDAP_NO_MEMORY;
@@ -850,8 +844,7 @@ put_vrFilter( BerElement *ber, const char *str_in )
 
 
 			default:
-				Debug( LDAP_DEBUG_TRACE, "put_vrFilter: simple\n",
-				    0, 0, 0 );
+				Debug0( LDAP_DEBUG_TRACE, "put_vrFilter: simple\n" );
 
 				balance = 1;
 				escape = 0;
@@ -896,8 +889,7 @@ put_vrFilter( BerElement *ber, const char *str_in )
 			break;
 
 		case /*'('*/ ')':
-			Debug( LDAP_DEBUG_TRACE, "put_vrFilter: end\n",
-				0, 0, 0 );
+			Debug0( LDAP_DEBUG_TRACE, "put_vrFilter: end\n" );
 			if ( ber_printf( ber, /*"["*/ "]" ) == -1 ) {
 				rc = -1;
 				goto done;
@@ -911,8 +903,7 @@ put_vrFilter( BerElement *ber, const char *str_in )
 			break;
 
 		default:	/* assume it's a simple type=value filter */
-			Debug( LDAP_DEBUG_TRACE, "put_vrFilter: default\n",
-				0, 0, 0 );
+			Debug0( LDAP_DEBUG_TRACE, "put_vrFilter: default\n" );
 			next = strchr( str, '\0' );
 			if ( put_simple_vrFilter( ber, str ) == -1 ) {
 				rc = -1;
@@ -954,8 +945,8 @@ put_vrFilter_list( BerElement *ber, char *str )
 	char	*next = NULL;
 	char	save;
 
-	Debug( LDAP_DEBUG_TRACE, "put_vrFilter_list \"%s\"\n",
-		str, 0, 0 );
+	Debug1( LDAP_DEBUG_TRACE, "put_vrFilter_list \"%s\"\n",
+		str );
 
 	while ( *str ) {
 		while ( *str && LDAP_SPACE( (unsigned char) *str ) ) {
@@ -988,8 +979,8 @@ put_simple_vrFilter(
 	ber_tag_t	ftype;
 	int		rc = -1;
 
-	Debug( LDAP_DEBUG_TRACE, "put_simple_vrFilter: \"%s\"\n",
-		str, 0, 0 );
+	Debug1( LDAP_DEBUG_TRACE, "put_simple_vrFilter: \"%s\"\n",
+		str );
 
 	str = LDAP_STRDUP( str );
 	if( str == NULL ) return -1;
