@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2005-2021 The OpenLDAP Foundation.
+ * Copyright 2005-2022 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,7 +29,7 @@
 #include <ac/unistd.h>
 #include "slap.h"
 #include "lutil.h"
-#include "config.h"
+#include "slap-config.h"
 
 struct ldif_tool {
 	Entry	**entries;			/* collected by bi_tool_entry_first() */
@@ -113,7 +113,7 @@ static int write_data( int fd, const char *spew, int len, int *save_errno );
 #endif /* !_WIN32 */
 
 /*
- * Left and Right "{num}" prefix to ordered RDNs ("olcDatabase={1}bdb").
+ * Left and Right "{num}" prefix to ordered RDNs ("olcDatabase={1}mdb").
  * IX_DN* are for LDAP RDNs, IX_FS* for their .ldif filenames.
  */
 #define IX_DNL	'{'
@@ -266,7 +266,7 @@ fullpath_alloc( struct berval *dest, const struct berval *dir, ber_len_t more )
 	dest->bv_val = s;
 	if ( s == NULL ) {
 		dest->bv_len = 0;
-		Debug( LDAP_DEBUG_ANY, "back-ldif: out of memory\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_ANY, "back-ldif: out of memory\n" );
 	} else {
 		s = lutil_strcopy( dest->bv_val, dir->bv_val );
 		*s++ = LDAP_DIRSEP[0];
@@ -472,22 +472,23 @@ ldif_read_file( const char *path, char **datap )
 				}
 				if ( crc1 != crc2 ) {
 					Debug( LDAP_DEBUG_ANY, "ldif_read_file: checksum error on \"%s\"\n",
-						path, 0, 0 );
+						path );
 					return rc;
 				}
 			}
 		}
-		Debug( LDAP_DEBUG_TRACE, "ldif_read_file: %s: \"%s\"\n", msg, path, 0 );
+		Debug( LDAP_DEBUG_TRACE, "ldif_read_file: %s: \"%s\"\n", msg, path );
 #endif /* LDAP_DEBUG */
 	} else {
+		char ebuf[128];
 		if ( res < 0 && errno == ENOENT ) {
 			Debug( LDAP_DEBUG_TRACE, "ldif_read_file: "
-				"no entry file \"%s\"\n", path, 0, 0 );
+				"no entry file \"%s\"\n", path );
 			rc = LDAP_NO_SUCH_OBJECT;
 		} else {
-			msg = res < 0 ? STRERROR( errno ) : "bad stat() size";
+			msg = res < 0 ? AC_STRERROR_R( errno, ebuf, sizeof(ebuf) ) : "bad stat() size";
 			Debug( LDAP_DEBUG_ANY, "ldif_read_file: %s for \"%s\"\n",
-				msg, path, 0 );
+				msg, path );
 			rc = LDAP_OTHER;
 		}
 		if ( data != NULL )
@@ -543,6 +544,7 @@ ldif_write_entry(
 	int rc = LDAP_OTHER, res, save_errno = 0;
 	int fd, entry_length;
 	char *entry_as_string, *tmpfname;
+	char ebuf[128];
 
 	if ( op->o_abandon )
 		return SLAPD_ABANDON;
@@ -551,7 +553,7 @@ ldif_write_entry(
 		save_errno = errno;
 		Debug( LDAP_DEBUG_ANY, "ldif_write_entry: %s \"%s\": %s\n",
 			"cannot create parent directory",
-			parentdir, STRERROR( save_errno ) );
+			parentdir, AC_STRERROR_R( save_errno, ebuf, sizeof(ebuf) ) );
 		*text = "internal error (cannot create parent directory)";
 		return rc;
 	}
@@ -561,7 +563,7 @@ ldif_write_entry(
 	if ( fd < 0 ) {
 		save_errno = errno;
 		Debug( LDAP_DEBUG_ANY, "ldif_write_entry: %s for \"%s\": %s\n",
-			"cannot create file", e->e_dn, STRERROR( save_errno ) );
+			"cannot create file", e->e_dn, AC_STRERROR_R( save_errno, ebuf, sizeof(ebuf) ) );
 		*text = "internal error (cannot create file)";
 
 	} else {
@@ -596,18 +598,18 @@ ldif_write_entry(
 		if ( res >= 0 ) {
 			if ( move_file( tmpfname, path->bv_val ) == 0 ) {
 				Debug( LDAP_DEBUG_TRACE, "ldif_write_entry: "
-					"wrote entry \"%s\"\n", e->e_name.bv_val, 0, 0 );
+					"wrote entry \"%s\"\n", e->e_name.bv_val );
 				rc = LDAP_SUCCESS;
 			} else {
 				save_errno = errno;
 				Debug( LDAP_DEBUG_ANY, "ldif_write_entry: "
 					"could not put entry file for \"%s\" in place: %s\n",
-					e->e_name.bv_val, STRERROR( save_errno ), 0 );
+					e->e_name.bv_val, AC_STRERROR_R( save_errno, ebuf, sizeof(ebuf) ) );
 				*text = "internal error (could not put entry file in place)";
 			}
 		} else if ( res == -1 ) {
 			Debug( LDAP_DEBUG_ANY, "ldif_write_entry: %s \"%s\": %s\n",
-				"write error to", tmpfname, STRERROR( save_errno ) );
+				"write error to", tmpfname, AC_STRERROR_R( save_errno, ebuf, sizeof(ebuf) ) );
 			*text = "internal error (write error to entry file)";
 		}
 
@@ -750,7 +752,7 @@ ldif_send_entry( Operation *op, SlapReply *rs, Entry *e, int scope )
 					sizeof(Entry *) * elen );
 				if ( entries == NULL ) {
 					Debug( LDAP_DEBUG_ANY,
-						"ldif_send_entry: out of memory\n", 0, 0, 0 );
+						"ldif_send_entry: out of memory\n" );
 					rc = LDAP_OTHER;
 					goto done;
 				}
@@ -758,6 +760,7 @@ ldif_send_entry( Operation *op, SlapReply *rs, Entry *e, int scope )
 				tl->entries = entries;
 			}
 			tl->entries[tl->ecount++] = e;
+			e->e_id = tl->ecount;
 			return rc;
 		}
 
@@ -806,6 +809,7 @@ ldif_readdir(
 {
 	int rc = LDAP_SUCCESS;
 	DIR *dir_of_path;
+	char ebuf[128];
 
 	*listp = NULL;
 	*fname_maxlenp = 0;
@@ -820,7 +824,7 @@ ldif_readdir(
 		if ( is_rootDSE || save_errno != ENOENT ) {
 			Debug( LDAP_DEBUG_ANY,
 				"=> ldif_search_entry: failed to opendir \"%s\": %s\n",
-				path->bv_val, STRERROR( save_errno ), 0 );
+				path->bv_val, AC_STRERROR_R( save_errno, ebuf, sizeof(ebuf) ) );
 			rc = LDAP_OTHER;
 			if ( rs != NULL )
 				rs->sr_text =
@@ -893,7 +897,7 @@ ldif_readdir(
 		if ( rc != LDAP_SUCCESS ) {
 			Debug( LDAP_DEBUG_ANY, "ldif_search_entry: %s \"%s\": %s\n",
 				"error reading directory", path->bv_val,
-				STRERROR( save_errno ) );
+				AC_STRERROR_R( save_errno, ebuf, sizeof(ebuf) ) );
 		}
 	}
 
@@ -927,7 +931,7 @@ ldif_search_entry(
 			 ber_dupbv( &ndn, &e->e_nname ) == NULL )
 		{
 			Debug( LDAP_DEBUG_ANY,
-				"ldif_search_entry: out of memory\n", 0, 0, 0 );
+				"ldif_search_entry: out of memory\n" );
 			rc = LDAP_OTHER;
 			goto done;
 		}
@@ -981,7 +985,7 @@ ldif_search_entry(
 								"(did someone just remove an entry file?)";
 						Debug( LDAP_DEBUG_ANY, "ldif_search_entry: "
 							"file listed in parent directory does not exist: "
-							"\"%s\"\n", fpath.bv_val, 0, 0 );
+							"\"%s\"\n", fpath.bv_val );
 						break;
 					}
 				}
@@ -1051,6 +1055,7 @@ ldif_prepare_create(
 	struct stat st;
 	Entry *parent = NULL;
 	int rc;
+	char ebuf[128];
 
 	if ( op->o_abandon )
 		return SLAPD_ABANDON;
@@ -1066,7 +1071,7 @@ ldif_prepare_create(
 	} else if ( errno != ENOENT ) {
 		Debug( LDAP_DEBUG_ANY,
 			"ldif_prepare_create: cannot stat \"%s\": %s\n",
-			dnpath->bv_val, STRERROR( errno ), 0 );
+			dnpath->bv_val, AC_STRERROR_R( errno, ebuf, sizeof(ebuf) ) );
 		rc = LDAP_OTHER;
 		*text = "internal error (cannot check entry file)";
 
@@ -1119,7 +1124,7 @@ ldif_prepare_create(
 		case LDAP_OTHER:
 			Debug( LDAP_DEBUG_ANY,
 				"ldif_prepare_create: cannot stat \"%s\" parent dir: %s\n",
-				ndn->bv_val, STRERROR( errno ), 0 );
+				ndn->bv_val, AC_STRERROR_R( errno, ebuf, sizeof(ebuf) ) );
 			*text = "internal error (cannot stat parent dir)";
 			break;
 		}
@@ -1379,6 +1384,7 @@ ldif_back_search( Operation *op, SlapReply *rs )
 	ldap_pvt_thread_rdwr_rlock(&li->li_rdwr);
 	rs->sr_err = search_tree( op, rs );
 	ldap_pvt_thread_rdwr_runlock(&li->li_rdwr);
+	rs->sr_ctrls = NULL;
 	send_ldap_result(op, rs);
 
 	return rs->sr_err;
@@ -1394,7 +1400,7 @@ ldif_back_add( Operation *op, SlapReply *rs )
 	char textbuf[SLAP_TEXT_BUFLEN];
 	int rc;
 
-	Debug( LDAP_DEBUG_TRACE, "ldif_back_add: \"%s\"\n", e->e_dn, 0, 0 );
+	Debug( LDAP_DEBUG_TRACE, "ldif_back_add: \"%s\"\n", e->e_dn );
 
 	rc = entry_schema_check( op, e, NULL, 0, 1, NULL,
 		&rs->sr_text, textbuf, sizeof( textbuf ) );
@@ -1423,7 +1429,7 @@ ldif_back_add( Operation *op, SlapReply *rs )
  send_res:
 	rs->sr_err = rc;
 	Debug( LDAP_DEBUG_TRACE, "ldif_back_add: err: %d text: %s\n",
-		rc, rs->sr_text ? rs->sr_text : "", 0 );
+		rc, rs->sr_text ? rs->sr_text : "" );
 	send_ldap_result( op, rs );
 	slap_graduate_commit_csn( op );
 	rs->sr_text = NULL;	/* remove possible pointer to textbuf */
@@ -1472,6 +1478,7 @@ ldif_back_delete( Operation *op, SlapReply *rs )
 	struct ldif_info *li = (struct ldif_info *) op->o_bd->be_private;
 	struct berval path;
 	int rc = LDAP_SUCCESS;
+	char ebuf[128];
 
 	if ( BER_BVISEMPTY( &op->o_csn )) {
 		struct berval csn;
@@ -1524,7 +1531,7 @@ ldif_back_delete( Operation *op, SlapReply *rs )
 
 	if ( rc == LDAP_OTHER ) {
 		Debug( LDAP_DEBUG_ANY, "ldif_back_delete: %s \"%s\": %s\n",
-			"cannot delete", path.bv_val, STRERROR( errno ) );
+			"cannot delete", path.bv_val, AC_STRERROR_R( errno, ebuf, sizeof(ebuf) ) );
 	}
 
 	SLAP_FREE( path.bv_val );
@@ -1550,6 +1557,7 @@ ldif_move_entry(
 	struct berval newpath;
 	char *parentdir = NULL, *trash;
 	int rc, rename_res;
+	char ebuf[128];
 
 	if ( same_ndn ) {
 		rc = LDAP_SUCCESS;
@@ -1603,11 +1611,10 @@ ldif_move_entry(
 			}
 
 			if ( rc != LDAP_SUCCESS ) {
-				char s[128];
-				snprintf( s, sizeof s, "%s (%s)", *text, STRERROR( errno ));
-				Debug( LDAP_DEBUG_ANY,
-					"ldif_move_entry: %s: \"%s\" -> \"%s\"\n",
-					s, op->o_req_dn.bv_val, entry->e_dn );
+				Debug(LDAP_DEBUG_ANY,
+				      "ldif_move_entry: %s (%s): \"%s\" -> \"%s\"\n",
+				      *text, AC_STRERROR_R(errno, ebuf, sizeof(ebuf)),
+				      op->o_req_dn.bv_val, entry->e_dn );
 			}
 		}
 
@@ -1704,6 +1711,30 @@ ldif_back_entry_get(
 	return rc;
 }
 
+static int
+ldif_back_entry_release_rw (
+	Operation *op,
+	Entry *e,
+	int rw )
+{
+	ID id = e->e_id;
+
+	/* only tool mode assigns valid IDs */
+	if ( id != 0 && id != NOID )
+	{
+		struct ldif_tool *tl = &((struct ldif_info *) op->o_bd->be_private)->li_tool;
+
+		id--;
+
+		assert( id < tl->ecount );
+		assert( e == tl->entries[id] );
+		tl->entries[id] = NULL;
+	}
+
+	entry_free( e );
+	return 0;
+}
+
 
 /* Slap tools */
 
@@ -1784,6 +1815,23 @@ ldif_tool_entry_first_x( BackendDB *be, struct berval *base, int scope, Filter *
 	return ldif_tool_entry_next( be );
 }
 
+static ID
+ldif_tool_dn2id_get( BackendDB *be, struct berval *dn )
+{
+	struct ldif_tool *tl = &((struct ldif_info *) be->be_private)->li_tool;
+
+	Operation op = {0};
+
+	op.o_bd = be;
+	op.o_req_dn = *dn;
+	op.o_req_ndn = *dn;
+	op.ors_scope = LDAP_SCOPE_BASE;
+	if ( search_tree( &op, NULL ) != LDAP_SUCCESS ) {
+		return NOID;
+	}
+	return tl->ecount;
+}
+
 static Entry *
 ldif_tool_entry_get( BackendDB *be, ID id )
 {
@@ -1793,7 +1841,6 @@ ldif_tool_entry_get( BackendDB *be, ID id )
 	--id;
 	if ( id < tl->ecount ) {
 		e = tl->entries[id];
-		tl->entries[id] = NULL;
 	}
 	return e;
 }
@@ -1826,6 +1873,76 @@ ldif_tool_entry_put( BackendDB *be, Entry *e, struct berval *text )
 	return NOID;
 }
 
+static ID
+ldif_tool_entry_modify( BackendDB *be, Entry *e, struct berval *text )
+{
+	int rc;
+	const char *errmsg = NULL;
+	struct berval path;
+	Operation op = {0};
+
+	op.o_bd = be;
+	ndn2path( &op, &e->e_nname, &path, 0 );
+	rc = ldif_write_entry( &op, e, &path, NULL, &errmsg );
+	SLAP_FREE( path.bv_val );
+	if ( rc == LDAP_SUCCESS )
+		return 1;
+
+	if ( errmsg == NULL && rc != LDAP_OTHER )
+		errmsg = ldap_err2string( rc );
+	if ( errmsg != NULL )
+		snprintf( text->bv_val, text->bv_len, "%s", errmsg );
+	return NOID;
+}
+
+static int
+ldif_tool_entry_delete( BackendDB *be, struct berval *ndn, struct berval *text )
+{
+	int rc = LDAP_SUCCESS;
+	const char *errmsg = NULL;
+	struct berval path;
+	Operation op = {0};
+
+	op.o_bd = be;
+	ndn2path( &op, ndn, &path, 0 );
+
+	ldif2dir_len( path );
+	ldif2dir_name( path );
+	if ( rmdir( path.bv_val ) < 0 ) {
+		switch ( errno ) {
+		case ENOTEMPTY:
+			rc = LDAP_NOT_ALLOWED_ON_NONLEAF;
+			break;
+		case ENOENT:
+			/* is leaf, go on */
+			break;
+		default:
+			rc = LDAP_OTHER;
+			errmsg = "internal error (cannot delete subtree directory)";
+			break;
+		}
+	}
+
+	if ( rc == LDAP_SUCCESS ) {
+		dir2ldif_name( path );
+		if ( unlink( path.bv_val ) < 0 ) {
+			rc = LDAP_NO_SUCH_OBJECT;
+			if ( errno != ENOENT ) {
+				rc = LDAP_OTHER;
+				errmsg = "internal error (cannot delete entry file)";
+			}
+		}
+	}
+
+	SLAP_FREE( path.bv_val );
+
+	if ( errmsg == NULL && rc != LDAP_OTHER )
+		errmsg = ldap_err2string( rc );
+	if ( errmsg != NULL )
+		snprintf( text->bv_val, text->bv_len, "%s", errmsg );
+	return rc;
+}
+
 
 /* Setup */
 
@@ -1860,7 +1977,7 @@ ldif_back_db_open( Backend *be, ConfigReply *cr )
 {
 	struct ldif_info *li = (struct ldif_info *) be->be_private;
 	if( BER_BVISEMPTY(&li->li_base_path)) {/* missing base path */
-		Debug( LDAP_DEBUG_ANY, "missing base path for back-ldif\n", 0, 0, 0);
+		Debug( LDAP_DEBUG_ANY, "missing base path for back-ldif\n" );
 		return 1;
 	}
 	return 0;
@@ -1910,6 +2027,7 @@ ldif_back_initialize( BackendInfo *bi )
 	bi->bi_connection_destroy = 0;
 
 	bi->bi_entry_get_rw = ldif_back_entry_get;
+	bi->bi_entry_release_rw = ldif_back_entry_release_rw;
 
 #if 0	/* NOTE: uncomment to completely disable access control */
 	bi->bi_access_allowed = slap_access_always_allowed;
@@ -1920,14 +2038,14 @@ ldif_back_initialize( BackendInfo *bi )
 	bi->bi_tool_entry_first = backend_tool_entry_first;
 	bi->bi_tool_entry_first_x = ldif_tool_entry_first_x;
 	bi->bi_tool_entry_next = ldif_tool_entry_next;
+	bi->bi_tool_dn2id_get = ldif_tool_dn2id_get;
 	bi->bi_tool_entry_get = ldif_tool_entry_get;
 	bi->bi_tool_entry_put = ldif_tool_entry_put;
+	bi->bi_tool_entry_modify = ldif_tool_entry_modify;
+	bi->bi_tool_entry_delete = ldif_tool_entry_delete;
 	bi->bi_tool_entry_reindex = 0;
 	bi->bi_tool_sync = 0;
 
-	bi->bi_tool_dn2id_get = 0;
-	bi->bi_tool_entry_modify = 0;
-
 	bi->bi_cf_ocs = ldifocs;
 
 	rc = config_register_schema( ldifcfg, ldifocs );
