@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2021 The OpenLDAP Foundation.
+ * Copyright 2001-2022 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -24,7 +24,7 @@
 
 #include <ldap_pvt.h>
 #include <ldap_pvt_thread.h>
-#include <avl.h>
+#include <ldap_avl.h>
 #include <slap.h>
 
 LDAP_BEGIN_DECL
@@ -132,6 +132,7 @@ typedef struct monitor_info_t {
 	AttributeDescription	*mi_ad_monitorUpdateRef;
 	AttributeDescription	*mi_ad_monitorRuntimeConfig;
 	AttributeDescription	*mi_ad_monitorSuperiorDN;
+	AttributeDescription	*mi_ad_monitorConnectionOpsAsync;
 
 	/*
 	 * Generic description attribute
@@ -315,6 +316,7 @@ typedef struct monitor_extra_t {
 		struct berval *modify );
 	monitor_entry_t * (*entrypriv_create)( void );
 	int (*register_subsys_late)( monitor_subsys_t *ms );
+	Entry * (*entry_get_unlocked)( struct berval *ndn );
 } monitor_extra_t;
 
 LDAP_END_DECL
