@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2021 The OpenLDAP Foundation.
+ * Copyright 2001-2022 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -77,7 +77,7 @@ monitor_subsys_rww_init(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_rww_init: "
 			"unable to get entry \"%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
+			ms->mss_ndn.bv_val );
 		return( -1 );
 	}
 
@@ -95,7 +95,7 @@ monitor_subsys_rww_init(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_subsys_rww_init: "
 				"unable to create entry \"cn=Read,%s\"\n",
-				ms->mss_ndn.bv_val, 0, 0 );
+				ms->mss_ndn.bv_val );
 			return( -1 );
 		}
 
@@ -120,7 +120,7 @@ monitor_subsys_rww_init(
 				"monitor_subsys_rww_init: "
 				"unable to add entry \"%s,%s\"\n",
 				monitor_rww[ i ].rdn.bv_val,
-				ms->mss_ndn.bv_val, 0 );
+				ms->mss_ndn.bv_val );
 			return( -1 );
 		}
 	
