@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2021 The OpenLDAP Foundation.
+ * Copyright 1999-2022 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Masarati.
  * Portions Copyright 2004 Mark Adamson.
@@ -93,7 +93,7 @@ backsql_attrlist_add( backsql_srch_info *bsi, AttributeDescription *ad )
 		
 		Debug( LDAP_DEBUG_TRACE, "==>backsql_attrlist_add(): "
 			"attribute \"%s\" is in list\n", 
-			an->an_name.bv_val, 0, 0 );
+			an->an_name.bv_val );
 		/*
 		 * We can live with strcmp because the attribute 
 		 * list has been normalized before calling be_search
@@ -104,7 +104,7 @@ backsql_attrlist_add( backsql_srch_info *bsi, AttributeDescription *ad )
 	}
 	
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_attrlist_add(): "
-		"adding \"%s\" to list\n", ad->ad_cname.bv_val, 0, 0 );
+		"adding \"%s\" to list\n", ad->ad_cname.bv_val );
 
 	an = (AttributeName *)bsi->bsi_op->o_tmprealloc( bsi->bsi_attrs,
 			sizeof( AttributeName ) * ( n_attrs + 2 ),
@@ -527,7 +527,7 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 
 	/* TimesTen */
 	Debug( LDAP_DEBUG_TRACE, "backsql_process_sub_filter(%s):\n",
-		at->bam_ad->ad_cname.bv_val, 0, 0 );
+		at->bam_ad->ad_cname.bv_val );
 	Debug(LDAP_DEBUG_TRACE, "   expr: '%s%s%s'\n", at->bam_sel_expr.bv_val,
 		at->bam_sel_expr_u.bv_val ? "' '" : "",
 		at->bam_sel_expr_u.bv_val ? at->bam_sel_expr_u.bv_val : "" );
@@ -558,7 +558,7 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 		Debug( LDAP_DEBUG_TRACE, 
 			"==>backsql_process_sub_filter(%s): "
 			"sub_initial=\"%s\"\n", at->bam_ad->ad_cname.bv_val,
-			f->f_sub_initial.bv_val, 0 );
+			f->f_sub_initial.bv_val );
 #endif /* BACKSQL_TRACE */
 
 		start = bsi->bsi_flt_where.bb_val.bv_len;
@@ -608,7 +608,7 @@ backsql_process_sub_filter( backsql_srch_info *bsi, Filter *f,
 		Debug( LDAP_DEBUG_TRACE, 
 			"==>backsql_process_sub_filter(%s): "
 			"sub_final=\"%s\"\n", at->bam_ad->ad_cname.bv_val,
-			f->f_sub_final.bv_val, 0 );
+			f->f_sub_final.bv_val );
 #endif /* BACKSQL_TRACE */
 
 		start = bsi->bsi_flt_where.bb_val.bv_len;
@@ -687,7 +687,7 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 	int 			done = 0;
 	int			rc = 0;
 
-	Debug( LDAP_DEBUG_TRACE, "==>backsql_process_filter()\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_TRACE, "==>backsql_process_filter()\n" );
 	if ( f->f_choice == SLAPD_FILTER_COMPUTED ) {
 		struct berval	flt;
 		char		*msg = NULL;
@@ -714,7 +714,7 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 		}
 
 		Debug( LDAP_DEBUG_TRACE, "backsql_process_filter(): "
-			"filter computed (%s)\n", msg, 0, 0 );
+			"filter computed (%s)\n", msg );
 		backsql_strfcat_x( &bsi->bsi_flt_where,
 				bsi->bsi_op->o_tmpmemctx, "b", &flt );
 		rc = 1;
@@ -817,7 +817,7 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 						"backsql_process_filter(): "
 						"unknown objectClass \"%s\" "
 						"in filter\n",
-						f->f_av_value.bv_val, 0, 0 );
+						f->f_av_value.bv_val );
 				bsi->bsi_status = LDAP_OTHER;
 				rc = -1;
 				goto done;
@@ -865,8 +865,7 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 			Debug( LDAP_DEBUG_TRACE,
 					"backsql_process_filter(): "
 					"illegal/unhandled filter "
-					"on objectClass attribute",
-					0, 0, 0 );
+					"on objectClass attribute" );
 			bsi->bsi_status = LDAP_OTHER;
 			rc = -1;
 			goto done;
@@ -1013,7 +1012,7 @@ backsql_process_filter( backsql_srch_info *bsi, Filter *f )
 
 	if ( vat == NULL ) {
 		/* search anyway; other parts of the filter
-		 * may succeeed */
+		 * may succeed */
 		backsql_strfcat_x( &bsi->bsi_flt_where,
 				bsi->bsi_op->o_tmpmemctx,
 				"l",
@@ -1075,7 +1074,7 @@ done:;
 
 	Debug( LDAP_DEBUG_TRACE,
 			"<==backsql_process_filter() %s\n",
-			rc == 1 ? "succeeded" : "failed", 0, 0);
+			rc == 1 ? "succeeded" : "failed" );
 
 	return rc;
 }
@@ -1185,7 +1184,7 @@ backsql_process_filter_attr( backsql_srch_info *bsi, Filter *f, backsql_at_map_r
 	struct berval		ordering = BER_BVC("<=");
 
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_process_filter_attr(%s)\n",
-		at->bam_ad->ad_cname.bv_val, 0, 0 );
+		at->bam_ad->ad_cname.bv_val );
 
 	/*
 	 * need to add this attribute to list of attrs to load,
@@ -1395,7 +1394,7 @@ equality_match:;
 	}
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_process_filter_attr(%s)\n",
-		at->bam_ad->ad_cname.bv_val, 0, 0 );
+		at->bam_ad->ad_cname.bv_val );
 
 	return 1;
 }
@@ -1411,7 +1410,7 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 
 	bsi->bsi_use_subtree_shortcut = 0;
 
-	Debug( LDAP_DEBUG_TRACE, "==>backsql_srch_query()\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_TRACE, "==>backsql_srch_query()\n" );
 	BER_BVZERO( &bsi->bsi_sel.bb_val );
 	BER_BVZERO( &bsi->bsi_sel.bb_val );
 	bsi->bsi_sel.bb_len = 0;
@@ -1637,7 +1636,7 @@ backsql_srch_query( backsql_srch_info *bsi, struct berval *query )
 	bsi->bsi_flt_where.bb_len = 0;
 	
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_srch_query() returns %s\n",
-		query->bv_val ? query->bv_val : "NULL", 0, 0 );
+		query->bv_val ? query->bv_val : "NULL" );
 	
 	return ( rc <= 0 ? 1 : 0 );
 }
@@ -1670,7 +1669,7 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 	bsi->bsi_status = LDAP_SUCCESS;
  
 	Debug( LDAP_DEBUG_TRACE, "==>backsql_oc_get_candidates(): oc=\"%s\"\n",
-			BACKSQL_OC_NAME( oc ), 0, 0 );
+			BACKSQL_OC_NAME( oc ) );
 
 	/* check for abandon */
 	if ( op->o_abandon ) {
@@ -1690,7 +1689,7 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 
 	if ( bsi->bsi_n_candidates == -1 ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
-			"unchecked limit has been overcome\n", 0, 0, 0 );
+			"unchecked limit has been overcome\n" );
 		/* should never get here */
 		assert( 0 );
 		bsi->bsi_status = LDAP_ADMINLIMIT_EXCEEDED;
@@ -1702,7 +1701,7 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 	if ( res ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
 			"error while constructing query for objectclass \"%s\"\n",
-			oc->bom_oc->soc_cname.bv_val, 0, 0 );
+			oc->bom_oc->soc_cname.bv_val );
 		/*
 		 * FIXME: need to separate errors from legally
 		 * impossible filters
@@ -1727,33 +1726,33 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 	if ( BER_BVISNULL( &query ) ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
 			"could not construct query for objectclass \"%s\"\n",
-			oc->bom_oc->soc_cname.bv_val, 0, 0 );
+			oc->bom_oc->soc_cname.bv_val );
 		bsi->bsi_status = LDAP_SUCCESS;
 		return BACKSQL_AVL_CONTINUE;
 	}
 
 	Debug( LDAP_DEBUG_TRACE, "Constructed query: %s\n", 
-			query.bv_val, 0, 0 );
+			query.bv_val );
 
 	rc = backsql_Prepare( bsi->bsi_dbh, &sth, query.bv_val, 0 );
 	bsi->bsi_op->o_tmpfree( query.bv_val, bsi->bsi_op->o_tmpmemctx );
 	BER_BVZERO( &query );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
-			"error preparing query\n", 0, 0, 0 );
+			"error preparing query\n" );
 		backsql_PrintErrors( bi->sql_db_env, bsi->bsi_dbh, sth, rc );
 		bsi->bsi_status = LDAP_OTHER;
 		return BACKSQL_AVL_CONTINUE;
 	}
 	
 	Debug( LDAP_DEBUG_TRACE, "id: '" BACKSQL_IDNUMFMT "'\n",
-		bsi->bsi_oc->bom_id, 0, 0 );
+		bsi->bsi_oc->bom_id );
 
 	rc = backsql_BindParamNumID( sth, 1, SQL_PARAM_INPUT,
 			&bsi->bsi_oc->bom_id );
 	if ( rc != SQL_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
-			"error binding objectclass id parameter\n", 0, 0, 0 );
+			"error binding objectclass id parameter\n" );
 		bsi->bsi_status = LDAP_OTHER;
 		return BACKSQL_AVL_CONTINUE;
 	}
@@ -1780,13 +1779,13 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 		}
 
 		Debug( LDAP_DEBUG_TRACE, "(base)dn: \"%s\"\n",
-				tmp_base_ndn, 0, 0 );
+				tmp_base_ndn );
 
 		rc = backsql_BindParamStr( sth, 2, SQL_PARAM_INPUT,
 				tmp_base_ndn, BACKSQL_MAX_DN_LEN );
 		if ( rc != SQL_SUCCESS ) {
          		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
-				"error binding base_ndn parameter\n", 0, 0, 0 );
+				"error binding base_ndn parameter\n" );
 			backsql_PrintErrors( bi->sql_db_env, bsi->bsi_dbh, 
 					sth, rc );
 			bsi->bsi_status = LDAP_OTHER;
@@ -1861,18 +1860,17 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 
 		if ( bsi->bsi_scope == LDAP_SCOPE_SUBORDINATE ) {
 			Debug( LDAP_DEBUG_TRACE, "(children)dn: \"%s\"\n",
-				tmp_base_ndn, 0, 0 );
+				tmp_base_ndn );
 		} else {
 			Debug( LDAP_DEBUG_TRACE, "(sub)dn: \"%s\"\n",
-				tmp_base_ndn, 0, 0 );
+				tmp_base_ndn );
 		}
 
 		rc = backsql_BindParamStr( sth, 2, SQL_PARAM_INPUT,
 				tmp_base_ndn, BACKSQL_MAX_DN_LEN );
 		if ( rc != SQL_SUCCESS ) {
 			Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
-				"error binding base_ndn parameter (2)\n",
-				0, 0, 0 );
+				"error binding base_ndn parameter (2)\n" );
 			backsql_PrintErrors( bi->sql_db_env, bsi->bsi_dbh, 
 					sth, rc );
 			bsi->bsi_status = LDAP_OTHER;
@@ -1885,12 +1883,12 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 		assert( !BER_BVISNULL( &bsi->bsi_base_id.eid_ndn ) );
 
 		Debug( LDAP_DEBUG_TRACE, "(one)id=" BACKSQL_IDFMT "\n",
-			BACKSQL_IDARG(bsi->bsi_base_id.eid_id), 0, 0 );
+			BACKSQL_IDARG(bsi->bsi_base_id.eid_id) );
 		rc = backsql_BindParamID( sth, 2, SQL_PARAM_INPUT,
 				&bsi->bsi_base_id.eid_id );
 		if ( rc != SQL_SUCCESS ) {
 			Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
-				"error binding base id parameter\n", 0, 0, 0 );
+				"error binding base id parameter\n" );
 			bsi->bsi_status = LDAP_OTHER;
 			return BACKSQL_AVL_CONTINUE;
 		}
@@ -1900,7 +1898,7 @@ backsql_oc_get_candidates( void *v_oc, void *v_bsi )
 	rc = SQLExecute( sth );
 	if ( !BACKSQL_SUCCESS( rc ) ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_oc_get_candidates(): "
-			"error executing query\n", 0, 0, 0 );
+			"error executing query\n" );
 		backsql_PrintErrors( bi->sql_db_env, bsi->bsi_dbh, sth, rc );
 		SQLFreeStmt( sth, SQL_DROP );
 		bsi->bsi_status = LDAP_OTHER;
@@ -1987,7 +1985,7 @@ cleanup:;
 	SQLFreeStmt( sth, SQL_DROP );
 
 	Debug( LDAP_DEBUG_TRACE, "<==backsql_oc_get_candidates(): %d\n",
-			n_candidates - bsi->bsi_n_candidates, 0, 0 );
+			n_candidates - bsi->bsi_n_candidates );
 
 	return ( bsi->bsi_n_candidates == -1 ? BACKSQL_AVL_STOP : BACKSQL_AVL_CONTINUE );
 }
@@ -2023,7 +2021,7 @@ backsql_search( Operation *op, SlapReply *rs )
 	if ( op->o_req_ndn.bv_len > BACKSQL_MAX_DN_LEN ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_search(): "
 			"search base length (%ld) exceeds max length (%d)\n", 
-			op->o_req_ndn.bv_len, BACKSQL_MAX_DN_LEN, 0 );
+			op->o_req_ndn.bv_len, BACKSQL_MAX_DN_LEN );
 		/*
 		 * FIXME: a LDAP_NO_SUCH_OBJECT could be appropriate
 		 * since it is impossible that such a long DN exists
@@ -2037,8 +2035,7 @@ backsql_search( Operation *op, SlapReply *rs )
 	sres = backsql_get_db_conn( op, &dbh );
 	if ( sres != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_TRACE, "backsql_search(): "
-			"could not get connection handle - exiting\n", 
-			0, 0, 0 );
+			"could not get connection handle - exiting\n" );
 		rs->sr_err = sres;
 		rs->sr_text = sres == LDAP_OTHER ?  "SQL-backend error" : NULL;
 		send_ldap_result( op, rs );
@@ -2184,7 +2181,7 @@ backsql_search( Operation *op, SlapReply *rs )
 		 * of entries matching LDAP query filter and scope (or at least 
 		 * candidates), and get the IDs. Do this in ID order for paging.
 		 */
-		avl_apply( bi->sql_oc_by_id, backsql_oc_get_candidates,
+		ldap_avl_apply( bi->sql_oc_by_id, backsql_oc_get_candidates,
 				&bsi, BACKSQL_AVL_STOP, AVL_INORDER );
 
 		/* check for abandon */
@@ -2291,7 +2288,7 @@ backsql_search( Operation *op, SlapReply *rs )
 			if ( rc != LDAP_SUCCESS ) {
 				Debug( LDAP_DEBUG_TRACE, "backsql_search(): "
 					"error %d in backsql_id2entry() "
-					"- skipping\n", rc, 0, 0 );
+					"- skipping\n", rc );
 				continue;
 			}
 			e = &user_entry;
@@ -2389,7 +2386,7 @@ backsql_search( Operation *op, SlapReply *rs )
 				Debug(LDAP_DEBUG_TRACE, 
 					"backsql_search(): "
 					"has_children failed( %d)\n", 
-					rc, 0, 0 );
+					rc );
 				rc = 1;
 				goto next_entry;
 			}
@@ -2456,7 +2453,7 @@ backsql_search( Operation *op, SlapReply *rs )
 				 * better stop
 				 */
 				Debug( LDAP_DEBUG_TRACE, "backsql_search(): "
-					"connection lost\n", 0, 0, 0 );
+					"connection lost\n" );
 				goto end_of_search;
 
 			case LDAP_SIZELIMIT_EXCEEDED:
@@ -2554,7 +2551,8 @@ done:;
 		op->ors_scope = LDAP_SCOPE_BASE;
 	}
 
-	Debug( LDAP_DEBUG_TRACE, "<==backsql_search()\n", 0, 0, 0 );
+	SQLTransact( SQL_NULL_HENV, dbh, SQL_ROLLBACK );
+	Debug( LDAP_DEBUG_TRACE, "<==backsql_search()\n" );
 
 	return rs->sr_err;
 }
@@ -2607,8 +2605,7 @@ backsql_entry_get(
 		/* find attribute values */
 		if ( is_entry_alias( bsi.bsi_e ) ) {
 			Debug( LDAP_DEBUG_ACL,
-				"<= backsql_entry_get: entry is an alias\n",
-				0, 0, 0 );
+				"<= backsql_entry_get: entry is an alias\n" );
 			rc = LDAP_ALIAS_PROBLEM;
 			goto return_results;
 		}
@@ -2616,8 +2613,7 @@ backsql_entry_get(
 
 		if ( is_entry_referral( bsi.bsi_e ) ) {
 			Debug( LDAP_DEBUG_ACL,
-				"<= backsql_entry_get: entry is a referral\n",
-				0, 0, 0 );
+				"<= backsql_entry_get: entry is a referral\n" );
 			rc = LDAP_REFERRAL;
 			goto return_results;
 		}
@@ -2625,8 +2621,7 @@ backsql_entry_get(
 		if ( oc && !is_entry_objectclass( bsi.bsi_e, oc, 0 ) ) {
 			Debug( LDAP_DEBUG_ACL,
 					"<= backsql_entry_get: "
-					"failed to find objectClass\n",
-					0, 0, 0 ); 
+					"failed to find objectClass\n" );
 			rc = LDAP_NO_SUCH_ATTRIBUTE;
 			goto return_results;
 		}
@@ -2748,7 +2743,7 @@ send_paged_response(
 
 	Debug(LDAP_DEBUG_ARGS,
 		"send_paged_response: lastid=0x%08lx nentries=%d\n", 
-		lastid ? *lastid : 0, rs->sr_nentries, NULL );
+		lastid ? *lastid : 0, rs->sr_nentries );
 
 	BER_BVZERO( &ctrl.ldctl_value );
 	ctrls[0] = &ctrl;
