@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2005-2021 The OpenLDAP Foundation.
+ * Copyright 2005-2022 The OpenLDAP Foundation.
  * Portions Copyright 2005 Pierangelo Masarati <ando@sys-net.it>
  * All rights reserved.
  *
@@ -31,7 +31,7 @@
 #include <ac/socket.h>
 
 #include "slap.h"
-#include "config.h"
+#include "slap-config.h"
 #include "lutil.h"
 #include "ldif.h"
 
@@ -790,9 +790,10 @@ static ConfigDriver rc_cf_gen;
 
 static ConfigTable rccfg[] = {
 	{ "retcode-parent", "dn",
-		2, 2, 0, ARG_MAGIC|ARG_DN|RC_PARENT, rc_cf_gen,
+		2, 2, 0, ARG_MAGIC|ARG_DN|ARG_QUOTE|RC_PARENT, rc_cf_gen,
 		"( OLcfgOvAt:20.1 NAME 'olcRetcodeParent' "
 			"DESC '' "
+			"EQUALITY distinguishedNameMatch "
 			"SYNTAX OMsDN SINGLE-VALUE )", NULL, NULL },
 	{ "retcode-item", "rdn> <retcode> <...",
 		3, 0, 0, ARG_MAGIC|RC_ITEM, rc_cf_gen,
@@ -806,6 +807,7 @@ static ConfigTable rccfg[] = {
 			(void *)offsetof(retcode_t, rd_indir),
 		"( OLcfgOvAt:20.3 NAME 'olcRetcodeInDir' "
 			"DESC '' "
+			"EQUALITY booleanMatch "
 			"SYNTAX OMsBoolean SINGLE-VALUE )", NULL, NULL },
 
 	{ "retcode-sleep", "sleeptime",
@@ -813,6 +815,7 @@ static ConfigTable rccfg[] = {
 			(void *)offsetof(retcode_t, rd_sleep),
 		"( OLcfgOvAt:20.4 NAME 'olcRetcodeSleep' "
 			"DESC '' "
+			"EQUALITY integerMatch "
 			"SYNTAX OMsInteger SINGLE-VALUE )", NULL, NULL },
 
 	{ NULL, NULL, 0, 0, 0, ARG_IGNORED }
@@ -949,7 +952,7 @@ rc_cf_gen( ConfigArgs *c )
 				"\"retcode-item <RDN> <retcode> [<text>]\": "
 				"missing args" );
 			Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return ARG_BAD_CONF;
 		}
 
@@ -961,7 +964,7 @@ rc_cf_gen( ConfigArgs *c )
 				"unable to normalize RDN \"%s\": %d",
 				c->argv[ 1 ], rc );
 			Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return ARG_BAD_CONF;
 		}
 
@@ -970,7 +973,7 @@ rc_cf_gen( ConfigArgs *c )
 				"value \"%s\" is not a RDN",
 				c->argv[ 1 ] );
 			Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return ARG_BAD_CONF;
 		}
 
@@ -981,7 +984,7 @@ rc_cf_gen( ConfigArgs *c )
 					"either \"retcode-parent\" "
 					"or \"suffix\" must be defined" );
 				Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-					c->log, c->cr_msg, 0 );
+					c->log, c->cr_msg );
 				return ARG_BAD_CONF;
 			}
 
@@ -1001,7 +1004,7 @@ rc_cf_gen( ConfigArgs *c )
 				"unable to parse return code \"%s\"",
 				c->argv[ 2 ] );
 			Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return ARG_BAD_CONF;
 		}
 
@@ -1064,7 +1067,7 @@ rc_cf_gen( ConfigArgs *c )
 								ops[ j ] );
 							ldap_charray_free( ops );
 							Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-								c->log, c->cr_msg, 0 );
+								c->log, c->cr_msg );
 							return ARG_BAD_CONF;
 						}
 					}
@@ -1077,7 +1080,7 @@ rc_cf_gen( ConfigArgs *c )
 						snprintf( c->cr_msg, sizeof(c->cr_msg),
 							"\"text\" already provided" );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 					ber_str2bv( &c->argv[ i ][ STRLENOF( "text=" ) ], 0, 1, &rdi.rdi_text );
@@ -1090,7 +1093,7 @@ rc_cf_gen( ConfigArgs *c )
 						snprintf( c->cr_msg, sizeof(c->cr_msg),
 							"\"matched\" already provided" );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 					ber_str2bv( &c->argv[ i ][ STRLENOF( "matched=" ) ], 0, 0, &dn );
@@ -1099,7 +1102,7 @@ rc_cf_gen( ConfigArgs *c )
 							"unable to prettify matched DN \"%s\"",
 							&c->argv[ i ][ STRLENOF( "matched=" ) ] );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 
@@ -1112,7 +1115,7 @@ rc_cf_gen( ConfigArgs *c )
 						snprintf( c->cr_msg, sizeof(c->cr_msg),
 							"\"ref\" already provided" );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 
@@ -1123,7 +1126,7 @@ rc_cf_gen( ConfigArgs *c )
 							"resultCode may cause slapd failures "
 							"related to internal checks" );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 					}
 
 					refs = ldap_str2charray( &c->argv[ i ][ STRLENOF( "ref=" ) ], " " );
@@ -1144,7 +1147,7 @@ rc_cf_gen( ConfigArgs *c )
 						snprintf( c->cr_msg, sizeof(c->cr_msg),
 							"\"sleeptime\" already provided" );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 
@@ -1153,7 +1156,7 @@ rc_cf_gen( ConfigArgs *c )
 							"unable to parse \"sleeptime=%s\"",
 							&c->argv[ i ][ STRLENOF( "sleeptime=" ) ] );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 
@@ -1165,7 +1168,7 @@ rc_cf_gen( ConfigArgs *c )
 						snprintf( c->cr_msg, sizeof(c->cr_msg),
 							"\"unsolicited\" already provided" );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 
@@ -1179,7 +1182,7 @@ rc_cf_gen( ConfigArgs *c )
 							snprintf( c->cr_msg, sizeof(c->cr_msg),
 								"unable to parse \"unsolicited\"" );
 							Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-								c->log, c->cr_msg, 0 );
+								c->log, c->cr_msg );
 							return ARG_BAD_CONF;
 						}
 
@@ -1206,7 +1209,7 @@ rc_cf_gen( ConfigArgs *c )
 						snprintf( c->cr_msg, sizeof(c->cr_msg),
 							"unknown flag \"%s\"", arg );
 						Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-							c->log, c->cr_msg, 0 );
+							c->log, c->cr_msg );
 						return ARG_BAD_CONF;
 					}
 
@@ -1215,7 +1218,7 @@ rc_cf_gen( ConfigArgs *c )
 						"unknown option \"%s\"",
 						c->argv[ i ] );
 					Debug( LDAP_DEBUG_CONFIG, "%s: retcode: %s\n",
-						c->log, c->cr_msg, 0 );
+						c->log, c->cr_msg );
 					return ARG_BAD_CONF;
 				}
 			}
@@ -1235,10 +1238,14 @@ rc_cf_gen( ConfigArgs *c )
 		}
 		*--next = '\0';
 		
-		for ( rdip = &rd->rd_item; *rdip; rdip = &(*rdip)->rdi_next )
-			/* go to last */ ;
+		/* We're marked X-ORDERED 'VALUES', valx might be valid */
+		for ( i = 0, rdip = &rd->rd_item;
+			*rdip && (c->valx < 0 || i < c->valx);
+			rdip = &(*rdip)->rdi_next, i++ )
+			/* go to position */ ;
 
 		
+		rdi.rdi_next = *rdip;
 		*rdip = ( retcode_item_t * )ch_malloc( sizeof( retcode_item_t ) );
 		*(*rdip) = rdi;
 
@@ -1514,7 +1521,7 @@ retcode_initialize( void )
 		code = register_at( retcode_at[ i ].desc, retcode_at[ i ].ad, 0 );
 		if ( code ) {
 			Debug( LDAP_DEBUG_ANY,
-				"retcode: register_at failed\n", 0, 0, 0 );
+				"retcode: register_at failed\n" );
 			return code;
 		}
 
@@ -1525,7 +1532,7 @@ retcode_initialize( void )
 		code = register_oc( retcode_oc[ i ].desc, retcode_oc[ i ].oc, 0 );
 		if ( code ) {
 			Debug( LDAP_DEBUG_ANY,
-				"retcode: register_oc failed\n", 0, 0, 0 );
+				"retcode: register_oc failed\n" );
 			return code;
 		}
 
