@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2020 The OpenLDAP Foundation.
+ * Copyright 1998-2022 The OpenLDAP Foundation.
  * Portions Copyright 1998-2003 Kurt D. Zeilenga.
  * Portions Copyright 2003 IBM Corporation.
  * All rights reserved.
@@ -17,7 +17,7 @@
  */
 /* ACKNOWLEDGEMENTS:
  * This work was initially developed by Kurt Zeilenga for inclusion
- * in OpenLDAP Software.  Additional signficant contributors include
+ * in OpenLDAP Software.  Additional significant contributors include
  *    Jong Hyuk Choi
  *    Hallvard B. Furuseth
  *    Howard Chu
@@ -39,6 +39,7 @@
 #include "ldif.h"
 
 tool_vars tool_globals;
+enum slaptool slapTool;
 
 #ifdef CSRIMALLOC
 static char *leakfilename;
@@ -91,6 +92,11 @@ usage( int tool, const char *progname )
 		options = " [-c]\n\t[-g] [-n databasenumber | -b suffix] [attr ...] [-q] [-t]\n";
 		break;
 
+	case SLAPMODIFY:
+		options = " [-c]\n\t[-g] [-n databasenumber | -b suffix]\n"
+			"\t[-l ldiffile] [-j linenumber] [-q] [-u] [-s] [-w]\n";
+		break;
+
 	case SLAPTEST:
 		options = " [-n databasenumber] [-u] [-Q]\n";
 		break;
@@ -145,25 +151,25 @@ parse_slapopt( int tool, int *mode )
 
 	} else if ( strncasecmp( optarg, "ssf", len ) == 0 ) {
 		if ( lutil_atou( &ssf, p ) ) {
-			Debug( LDAP_DEBUG_ANY, "unable to parse ssf=\"%s\".\n", p, 0, 0 );
+			Debug( LDAP_DEBUG_ANY, "unable to parse ssf=\"%s\".\n", p );
 			return -1;
 		}
 
 	} else if ( strncasecmp( optarg, "transport_ssf", len ) == 0 ) {
 		if ( lutil_atou( &transport_ssf, p ) ) {
-			Debug( LDAP_DEBUG_ANY, "unable to parse transport_ssf=\"%s\".\n", p, 0, 0 );
+			Debug( LDAP_DEBUG_ANY, "unable to parse transport_ssf=\"%s\".\n", p );
 			return -1;
 		}
 
 	} else if ( strncasecmp( optarg, "tls_ssf", len ) == 0 ) {
 		if ( lutil_atou( &tls_ssf, p ) ) {
-			Debug( LDAP_DEBUG_ANY, "unable to parse tls_ssf=\"%s\".\n", p, 0, 0 );
+			Debug( LDAP_DEBUG_ANY, "unable to parse tls_ssf=\"%s\".\n", p );
 			return -1;
 		}
 
 	} else if ( strncasecmp( optarg, "sasl_ssf", len ) == 0 ) {
 		if ( lutil_atou( &sasl_ssf, p ) ) {
-			Debug( LDAP_DEBUG_ANY, "unable to parse sasl_ssf=\"%s\".\n", p, 0, 0 );
+			Debug( LDAP_DEBUG_ANY, "unable to parse sasl_ssf=\"%s\".\n", p );
 			return -1;
 		}
 
@@ -200,13 +206,13 @@ parse_slapopt( int tool, int *mode )
 			} else if ( strcasecmp( p, "no" ) == 0 ) {
 				*mode |= SLAP_TOOL_NO_SCHEMA_CHECK;
 			} else {
-				Debug( LDAP_DEBUG_ANY, "unable to parse schema-check=\"%s\".\n", p, 0, 0 );
+				Debug( LDAP_DEBUG_ANY, "unable to parse schema-check=\"%s\".\n", p );
 				return -1;
 			}
 			break;
 
 		default:
-			Debug( LDAP_DEBUG_ANY, "schema-check meaningless for tool.\n", 0, 0, 0 );
+			Debug( LDAP_DEBUG_ANY, "schema-check meaningless for tool.\n" );
 			break;
 		}
 
@@ -218,17 +224,18 @@ parse_slapopt( int tool, int *mode )
 			} else if ( strcasecmp( p, "no" ) == 0 ) {
 				*mode &= ~SLAP_TOOL_VALUE_CHECK;
 			} else {
-				Debug( LDAP_DEBUG_ANY, "unable to parse value-check=\"%s\".\n", p, 0, 0 );
+				Debug( LDAP_DEBUG_ANY, "unable to parse value-check=\"%s\".\n", p );
 				return -1;
 			}
 			break;
 
 		default:
-			Debug( LDAP_DEBUG_ANY, "value-check meaningless for tool.\n", 0, 0, 0 );
+			Debug( LDAP_DEBUG_ANY, "value-check meaningless for tool.\n" );
 			break;
 		}
 
-	} else if ( strncasecmp( optarg, "ldif-wrap", len ) == 0 ) {
+	} else if ( ( strncasecmp( optarg, "ldif_wrap", len ) == 0 ) ||
+			( strncasecmp( optarg, "ldif-wrap", len ) == 0 ) ) {
 		switch ( tool ) {
 		case SLAPCAT:
 			if ( strcasecmp( p, "no" ) == 0 ) {
@@ -237,7 +244,7 @@ parse_slapopt( int tool, int *mode )
 			} else {
 				unsigned int u;
 				if ( lutil_atou( &u, p ) ) {
-					Debug( LDAP_DEBUG_ANY, "unable to parse ldif-wrap=\"%s\".\n", p, 0, 0 );
+					Debug( LDAP_DEBUG_ANY, "unable to parse ldif_wrap=\"%s\".\n", p );
 					return -1;
 				}
 				ldif_wrap = (ber_len_t)u;
@@ -245,7 +252,7 @@ parse_slapopt( int tool, int *mode )
 			break;
 
 		default:
-			Debug( LDAP_DEBUG_ANY, "ldif-wrap meaningless for tool.\n", 0, 0, 0 );
+			Debug( LDAP_DEBUG_ANY, "ldif-wrap meaningless for tool.\n" );
 			break;
 		}
 
@@ -293,6 +300,8 @@ slap_tool_init(
 	ldif_debug = slap_debug;
 #endif
 	ldap_syslog = 0;
+	/* make sure libldap gets init'd */
+	ldap_set_option( NULL, LDAP_OPT_DEBUG_LEVEL, &slap_debug );
 
 #ifdef CSRIMALLOC
 	leakfilename = malloc( strlen( progname ) + STRLENOF( ".leak" ) + 1 );
@@ -323,6 +332,10 @@ slap_tool_init(
 		mode |= SLAP_TOOL_READMAIN | SLAP_TOOL_READONLY;
 		break;
 
+	case SLAPMODIFY:
+		options = "b:cd:f:F:gj:l:n:o:qsS:uvw";
+		break;
+
 	case SLAPSCHEMA:
 		options = "a:b:cd:f:F:gH:l:n:o:s:v";
 		mode |= SLAP_TOOL_READMAIN | SLAP_TOOL_READONLY;
@@ -357,7 +370,7 @@ slap_tool_init(
 	while ( (i = getopt( argc, argv, options )) != EOF ) {
 		switch ( i ) {
 		case 'a':
-			filterstr = ch_strdup( optarg );
+			filterstr = optarg;
 			break;
 
 		case 'b':
@@ -394,11 +407,11 @@ slap_tool_init(
 			break;
 
 		case 'f':	/* specify a conf file */
-			conffile = ch_strdup( optarg );
+			conffile = optarg;
 			break;
 
 		case 'F':	/* specify a conf dir */
-			confdir = ch_strdup( optarg );
+			confdir = optarg;
 			break;
 
 		case 'g':	/* disable subordinate glue */
@@ -459,7 +472,7 @@ slap_tool_init(
 			break;
 
 		case 'l':	/* LDIF file */
-			ldiffile = ch_strdup( optarg );
+			ldiffile = optarg;
 			break;
 
 		case 'M':
@@ -516,6 +529,7 @@ slap_tool_init(
 		case 's':
 			switch ( tool ) {
 			case SLAPADD:
+			case SLAPMODIFY:
 				/* no schema check */
 				mode |= SLAP_TOOL_NO_SCHEMA_CHECK;
 				break;
@@ -524,7 +538,7 @@ slap_tool_init(
 			case SLAPSCHEMA:
 				/* dump subtree */
 				ch_free( subtree );
-				subtree = ch_strdup( optarg );
+				subtree = optarg;
 				break;
 			}
 			break;
@@ -596,6 +610,7 @@ slap_tool_init(
 	switch ( tool ) {
 	case SLAPADD:
 	case SLAPCAT:
+	case SLAPMODIFY:
 	case SLAPSCHEMA:
 		if ( ( argc != optind ) || (dbnum >= 0 && base.bv_val != NULL ) ) {
 			usage( tool, progname );
@@ -657,6 +672,7 @@ slap_tool_init(
 	 * initialize stuff and figure out which backend we're dealing with
 	 */
 
+	slapTool = tool;
 	rc = slap_init( mode, progname );
 	if ( rc != 0 ) {
 		fprintf( stderr, "%s: slap_init failed!\n", progname );
@@ -695,6 +711,7 @@ slap_tool_init(
 	case SLAPADD:
 	case SLAPCAT:
 	case SLAPINDEX:
+	case SLAPMODIFY:
 	case SLAPSCHEMA:
 		if ( !nbackends ) {
 			fprintf( stderr, "No databases found "
@@ -745,9 +762,6 @@ slap_tool_init(
 			fprintf( stderr, "Invalid filter '%s'\n", filterstr );
 			exit( EXIT_FAILURE );
 		}
-
-		ch_free( filterstr );
-		filterstr = NULL;
 	}
 
 	if( subtree ) {
@@ -794,7 +808,7 @@ slap_tool_init(
 			break;
 		}
 
-		/* If the named base is a glue master, operate on the
+		/* If the named base is a glue primary, operate on the
 		 * entire context
 		 */
 		if ( SLAP_GLUE_INSTANCE( be ) ) {
@@ -822,7 +836,7 @@ slap_tool_init(
 				continue;
 
 		/* If just doing the first by default and it is a
-		 * glue subordinate, find the master.
+		 * glue subordinate, find the primary.
 		 */
 			if ( SLAP_GLUE_SUBORDINATE(be) ) {
 				nosubordinates = 1;
@@ -841,7 +855,7 @@ slap_tool_init(
 			Debug( LDAP_DEBUG_ANY,
 				"The first database does not allow %s;"
 				" using the first available one (%d)\n",
-				progname, dbnum, 0 );
+				progname, dbnum );
 		}
 
 	} else if ( dbnum >= nbackends ) {
@@ -886,20 +900,6 @@ startup:;
 	mal_leaktrace(1);
 #endif
 
-	if ( conffile != NULL ) {
-		ch_free( conffile );
-		conffile = NULL;
-	}
-
-	if ( confdir != NULL ) {
-		ch_free( confdir );
-		confdir = NULL;
-	}
-
-	if ( ldiffile != NULL ) {
-		ch_free( ldiffile );
-		ldiffile = NULL;
-	}
 
 	/* slapdn doesn't specify a backend to startup */
 	if ( !dryrun && tool != SLAPDN ) {
